/**
 * Module dependencies.
 */
var util = require('util')
  , OAuth2Strategy = require('passport-oauth').OAuth2Strategy;


/**
 * `Strategy` constructor.
 *
 * The Dwolla authentication strategy authenticates requests by delegating to
 * Dwolla using the OAuth 2.0 protocol.
 *
 * Applications must supply a `verify` callback which accepts an `accessToken`,
 * `refreshToken` and service-specific `profile`, and then calls the `done`
 * callback supplying a `user`, which should be set to `false` if the
 * credentials are not valid.  If an exception occured, `err` should be set.
 *
 * Options:
 *   - `clientID`      your Dwolla application's client id
 *   - `clientSecret`  your Dwolla application's client secret
 *   - `callbackURL`   URL to which Dwolla will redirect the user after granting authorization
 *
 * Examples:
 *
 *     passport.use(new DwollaStrategy({
 *         clientID: '123-456-789',
 *         clientSecret: 'shhh-its-a-secret'
 *         callbackURL: 'https://www.example.net/auth/dwolla/callback'
 *       },
 *       function(accessToken, refreshToken, profile, done) {
 *         User.findOrCreate(..., function (err, user) {
 *           done(err, user);
 *         });
 *       }
 *     ));
 *
 * @param {Object} options
 * @param {Function} verify
 * @api public
 */
function Strategy(options, verify) {
  options = options || {};
  options.authorizationURL = options.authorizationURL || 'https://www.dwolla.com/oauth/v2/authenticate';
  options.tokenURL = options.tokenURL || 'https://www.dwolla.com/oauth/v2/token';
  
  OAuth2Strategy.call(this, options, verify);
  this.name = 'dwolla';
  
  this._oauth2.setAccessTokenName("oauth_token");
}

/**
 * Inherit from `OAuth2Strategy`.
 */
util.inherits(Strategy, OAuth2Strategy);


/**
 * Retrieve user profile from Dwolla.
 *
 * This function constructs a normalized profile, with the following properties:
 *
 *   - `provider`         always set to `dwolla`
 *   - `id`
 *   - `username`
 *   - `displayName`
 *
 * @param {String} accessToken
 * @param {Function} done
 * @api protected
 */
Strategy.prototype.userProfile = function(accessToken, done) {
  // TODO: Fetch users resource or accounts resoruce.  Not sure...
  this._oauth2.getProtectedResource('https://www.dwolla.com/oauth/rest/accountapi/accountinformation', accessToken, function (err, body, res) {
    if (err) { return done(err); }
    
    try {
      var json = JSON.parse(body);
      
      var profile = { provider: 'dwolla' };
      profile.id = json['Id'];
      profile.displayName = json['Name'];
      
      done(null, profile);
    } catch(e) {
      done(e);
    }
  });
}


/**
 * Expose `Strategy`.
 */
module.exports = Strategy;
